@@ -22,3 +22,16 @@
 
 """
 """
+
+__version__ = '0.9.4'
+
+setproctitle = lambda v: NotImplemented
+
+try:
+    from setproctitle import setproctitle
+except ImportError:
+    try:
+        from procname import setprocname as setproctitle
+    except ImportError:
+        pass
+
