@@ -1,3 +1,4 @@
+#!/usr/bin/env python
 # Copyright (c) 2008, Donovan Preston
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,16 +24,27 @@
 """spawning_child.py
 """
 
-from eventlet import api, coros, greenio, wsgi
+import eventlet
+import eventlet.event
+import eventlet.greenio
+import eventlet.hubs
+import eventlet.wsgi
 
-import optparse, os, signal, socket, sys, time
+import errno
+import optparse
+import os
+import signal
+import socket
+import sys
+import time
 
-from paste.deploy import loadwsgi
-
-from spawning import reloader_dev
-
-import simplejson
+import spawning.util
+from spawning import setproctitle, reloader_dev
 
+try:
+    import json
+except ImportError:
+    import simplejson as json
 
 class FigleafCoverage(object):
     def __init__(self, app):
@@ -59,10 +71,9 @@ class FigleafCoverage(object):
 class ExitChild(Exception):
     pass
 
-
 def read_pipe_and_die(the_pipe, server_coro):
     try:
-        api.trampoline(the_pipe, read=True)
+        eventlet.hubs.trampoline(the_pipe, read=True)
         os.read(the_pipe, 1)
     except socket.error:
         pass
@@ -70,27 +81,7 @@ def read_pipe_and_die(the_pipe, server_coro):
         os.close(the_pipe)
     except socket.error:
         pass
-    api.switch(server_coro, exc=ExitChild)
-
-
-class ExecuteInThreadpool(object):
-    def __init__(self, app):
-        self.app = app
-
-    def __call__(self, env, start_response, exc_info=None):
-        from eventlet import tpool
-        head = []
-        body = []
-        def _start_response(status, headers, exc_info=None):
-            head[:] = status, headers, exc_info
-            return body.append
-
-        def get_list():
-            return list(self.app(env, _start_response))
-
-        result = tpool.execute(get_list)
-        start_response(*head)
-        return result
+    return server_coro.throw(ExitChild)
 
 
 def deadman_timeout(signum, frame):
@@ -98,23 +89,35 @@ def deadman_timeout(signum, frame):
         os.getpid(), )
     os.kill(os.getpid(), signal.SIGKILL)
 
+def tpool_wsgi(app):
+    from eventlet import tpool
+    def tpooled_application(e, s):
+        result = tpool.execute(app, e, s)
+        # return builtins directly
+        if isinstance(result, (basestring, list, tuple)):
+            return result
+        else:
+            # iterators might execute code when iterating over them,
+            # so we wrap them in a Proxy object so every call to
+            # next() goes through tpool
+            return tpool.Proxy(result)
+    return tpooled_application
 
-def serve_from_child(sock, config):
+
+def serve_from_child(sock, config, controller_pid):
     threads = config.get('threadpool_workers', 0)
-    wsgi_application = api.named(config['app_factory'])(config)
+    wsgi_application = spawning.util.named(config['app_factory'])(config)
 
     if config.get('coverage'):
         wsgi_application = FigleafCoverage(wsgi_application)
 
     if threads > 1:
-        wsgi_application = ExecuteInThreadpool(wsgi_application)
+        # proxy calls of the application through tpool
+        wsgi_application = tpool_wsgi(wsgi_application)
     elif threads != 1:
         print "(%s) not using threads, installing eventlet cooperation monkeypatching" % (
             os.getpid(), )
-        from eventlet import util
-        util.wrap_socket_with_coroutine_socket()
-        #util.wrap_pipes_with_coroutine_pipes()
-        #util.wrap_threading_local_with_coro_local()
+        eventlet.patcher.monkey_patch(all=False, socket=True)
 
     host, port = sock.getsockname()
 
@@ -122,10 +125,23 @@ def serve_from_child(sock, config):
     if access_log_file is not None:
         access_log_file = open(access_log_file, 'a')
 
-    server_event = coros.event()
+    max_age = 0
+    if config.get('max_age'):
+        max_age = int(config.get('max_age'))
+
+    server_event = eventlet.event.Event()
+    http_version = config.get('no_keepalive') and 'HTTP/1.0' or 'HTTP/1.1'
     try:
-        wsgi.server(
-            sock, wsgi_application, log=access_log_file, server_event=server_event)
+        wsgi_args = (sock, wsgi_application)
+        wsgi_kwargs = {'log' : access_log_file, 'server_event' : server_event, 'max_http_version' : http_version}
+        if config.get('no_keepalive'):
+            wsgi_kwargs.update({'keepalive' : False})
+        if max_age:
+            wsgi_kwargs.update({'timeout_value' : True})
+            eventlet.with_timeout(max_age, eventlet.wsgi.server, *wsgi_args,
+                    **wsgi_kwargs)
+        else:
+            eventlet.wsgi.server(*wsgi_args, **wsgi_kwargs)
     except KeyboardInterrupt:
         pass
     except ExitChild:
@@ -143,17 +159,24 @@ def serve_from_child(sock, config):
     server = server_event.wait()
 
     last_outstanding = None
+    if server.outstanding_requests:
+        ## Let's tell our parent that we're dying
+        try:
+            os.kill(controller_pid, signal.SIGUSR1)
+        except OSError, e:
+            if not e.errno == errno.ESRCH:
+                raise
+
     while server.outstanding_requests:
         if last_outstanding != server.outstanding_requests:
             print "(%s) %s requests remaining, waiting... (timeout after %s)" % (
                 os.getpid(), server.outstanding_requests, config['deadman_timeout'])
         last_outstanding = server.outstanding_requests
-        api.sleep(0.1)
+        eventlet.sleep(0.1)
 
     print "(%s) *** Child exiting: all requests completed at %s" % (
         os.getpid(), time.asctime())
 
-
 def main():
     parser = optparse.OptionParser()
     parser.add_option("-r", "--reload",
@@ -170,30 +193,33 @@ def main():
 
     controller_pid, httpd_fd, death_fd, factory_qual, factory_args = args
     controller_pid = int(controller_pid)
-    config = api.named(factory_qual)(simplejson.loads(factory_args))
+    config = spawning.util.named(factory_qual)(json.loads(factory_args))
+
+    setproctitle("spawn: child (%s)" % ", ".join(config.get("args")))
 
     ## Set up the reloader
-    if options.reload:
+    if config.get('reload'):
         watch = config.get('watch', None)
         if watch:
             watching = ' and %s' % watch
         else:
             watching = ''
         print "(%s) reloader watching sys.modules%s" % (os.getpid(), watching)
-        api.spawn(
+        eventlet.spawn(
             reloader_dev.watch_forever, controller_pid, 1, watch)
 
     ## The parent will catch sigint and tell us to shut down
     signal.signal(signal.SIGINT, signal.SIG_IGN)
-    api.spawn(read_pipe_and_die, int(death_fd), api.getcurrent())
+    ## Expect a SIGHUP when we want the child to die
+    signal.signal(signal.SIGHUP, lambda *a, **kw: exit(0))
+    eventlet.spawn(read_pipe_and_die, int(death_fd), eventlet.getcurrent())
 
     ## Make the socket object from the fd given to us by the controller
-    sock = greenio.GreenSocket(
+    sock = eventlet.greenio.GreenSocket(
         socket.fromfd(int(httpd_fd), socket.AF_INET, socket.SOCK_STREAM))
 
     serve_from_child(
-        sock, config)
-
+        sock, config, controller_pid)
 
 if __name__ == '__main__':
     main()
